import os.path


class KDFile:
    def __init__(self, header, data):
        """
        初始化方法，接收一个头部字典和一个数据列表作为参数
        """
        self.header = header
        self.data = data

    def __str__(self):
        """
        返回 KDFile 对象的字符串表示
        """
        return f"Header: {self.header}\nData: {self.data}"

    def get_kd_data(self):
        """
        返回 KDDate 对象的数据列表
        """
        unit = self.header["unit"]
        temperature = self.header["temperature"]
        figure_size = self.header["figure_size"]
        color_theme = self.header["color_theme"]
        font_family = self.header["font_family"]
        save_image = self.header["save_image"]
        num_data = self.data
        # 本地导包
        from kd_data import KDData

        kd_data = KDData(unit, temperature, figure_size, color_theme, font_family, save_image, num_data)
        return kd_data


class KDFileParser:
    def __init__(self, filepath):
        """
        初始化方法，接收一个文件路径作为参数
        """
        self.filepath = filepath

    def parse(self):
        """
        解析符合 .kd 文件格式的文件，返回一个 KDFile 对象
        """
        header, data = self._parse_file()
        return KDFile(header, data)

    def _validate(self):
        """
        验证符合上述格式的 KD 文件是否存在数据行。
        如果存在数据行，返回 True；否则返回 False。
        """
        # 确保文件后缀为 .kd
        if not self.filepath.endswith('.kd'):
            print(f"{self.filepath} 不是一个 KD 文件")
            return False

        # 确保文件存在
        if not os.path.isfile(self.filepath):
            print(f"{self.filepath} 文件不存在")
            return False

        # 验证是否存在数据行
        with open(self.filepath, 'r', encoding="utf-8") as f:
            begin_found = False
            for line in f:
                line = line.strip()
                if line == 'BEGIN':
                    begin_found = True
                elif line == 'END':
                    return begin_found

        return False  # 没有找到 BEGIN 和 END 标记，返回 False

    def _parse_file(self):
        """
        解析文件，返回头部字典和数据列表
        """
        if not self._validate():
            raise ValueError('Invalid KD file format')

        header = {'unit': None,
                  'temperature': None,
                  'figure_size': None,
                  'color_theme': None,
                  'font_family': None,
                  'save_image': None}

        data = []

        with open(self.filepath, 'r', encoding="utf-8") as f:
            lines = f.readlines()

        # 解析文件头
        for line in lines[:6]:
            key, value = line[2:].strip().split(' = ')
            if key == 'FIGURE_SIZE':
                width, height = map(int, value.split(','))
                header[key.lower()] = (width, height)
            else:
                header[key.lower()] = value

        # 解析数据行
        data_started = False
        for line in lines:
            if data_started:
                if line.strip() == 'END':
                    break
                x_str, y_str = line.strip().split(',')
                x, y = float(x_str), float(y_str)
                data.append((x, y))
            elif line.strip() == 'BEGIN':
                data_started = True

        return header, data



import numpy as np


class KDData:

    def __init__(self, unit, temperature, figure_size, color_theme, font_family, save_image, num_data):
        self.unit = unit
        self.temperature = temperature
        self.figure_size = figure_size
        self.color_theme = color_theme
        self.font_family = font_family
        self.save_image = save_image
        self.num_Data = num_data

    def __str__(self):
        return f"KDData(unit={self.unit}, temperature={self.temperature}, figure_size={self.figure_size}, color_theme={self.color_theme}, font_family={self.font_family}, save_image={self.save_image}, num_data={self.num_Data})"

    def get_num_x(self):
        """
        Returns the number of x-axis values
        """
        x = []
        for item in self.num_Data:
            x.append(item[0])

        return x

    def get_num_y(self):
        """
        Returns the number of y-axis values
        """
        y = []
        for item in self.num_Data:
            y.append(item[1])

        return y

    def get_numpy_array(self):
        """
        Returns the data as a numpy array
        """
        return np.array([self.get_num_x(), self.get_num_y()])




from setuptools import find_packages, setup

setup(
    name='kimariDraw',
    version='1.1',
    description='The application is used to draw the Energy Profile Map',
    author='kimariyb',
    author_email='kimariyb@163.com',
    url='https://github.com/kimariyb/kimariDraw',
    install_requires=[
        'numpy',
        'matplotlib',
    ],
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'kimariDraw = kimariDraw.__main__:main'     
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License"
    ],
    python_requires='>=3.9'
)


import numpy as np
import matplotlib.pyplot as plt

from kd_data import KDData


def kd_draw(data: KDData, save_name='figure'):
    """
    使用 matplotlib 绘制能量折线图
    """
    # 设置全局字体
    plt.rcParams['font.family'] = data.font_family
    plt.rcParams['font.weight'] = 'bold'
    plt.rcParams['axes.linewidth'] = 1.5

    num_x = data.get_num_x()
    num_y = data.get_num_y()

    # 创建画布和子图对象
    fig, ax = plt.subplots(figsize=data.figure_size)

    # 设置 x 轴和 y 轴的范围
    ax.set_xlim(0, np.max(num_x) + 1)
    ax.set_ylim(np.min(num_y) * 0.9, np.max(num_y) * 1.1)

    # 绘制平台
    # 绘制数据点
    # for i in range(len(num_x)):
    #    ax.scatter(num_x[i], num_y[i])

    # 在每个数据点上绘制长度为0.4的水平线
    for i, (x, y) in enumerate(zip(num_x, num_y)):
        ax.plot([x - 0.2, x + 0.2], [y, y], color='black', linewidth=3)
        ax.text(x, y * 1.01, f"{y:.1f}", ha='center', va='bottom', fontweight='bold', fontsize=10)

    for i in range(len(num_x) - 1):
        ax.plot([num_x[i]+0.2, num_x[i+1]-0.2], [num_y[i], num_y[i+1]],  color='black', linewidth=1, linestyle='--')

    # 设置 x 轴和 y 轴标签
    y_label = "Free Energy" + f" ({data.unit})"
    ax.set_xlabel("Reaction Coordinate", fontweight='bold', fontsize=14)
    ax.set_ylabel(y_label, fontweight='bold', fontsize=14)

    # 设置 x 轴和 y 轴坐标
    ax.tick_params(axis='x', color='white')
    ax.tick_params(axis='x', labelcolor='white')

    # 设置标题
    ax.set_title("Chemical Energy Profile", fontweight='bold', fontsize=16)

    # 保存路径
    save_fig_url = save_name + "." + data.save_image
    plt.savefig(save_fig_url, dpi=700)

    # 显示图像
    plt.show()



import argparse
from kd_parser import *
from kd_plot import *
from kd_data import *

def parse_args():
    parser = argparse.ArgumentParser(description='Draw energy profile plot using matplotlib')
    parser.add_argument('input_file', type=str, help='path to input data file')
    parser.add_argument('-s', '--save_name', type=str, default='figure', help='name of the output image file')
    return parser.parse_args()

def main():
    args = parse_args()
    # 读取数据    
    data = KDFileParser(args.input_file).parse().get_kd_data()
    # 判断 save_name 是否为空
    if args.save_name == None:
        kd_draw(data)
    else:
        kd_draw(data, args.save_name)

if __name__ == "__main__":
    main()